package cn.ecpark.service.usergw.config;

import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.NettyWriteResponseFilter;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.cloud.gateway.route.RouteDefinitionLocator;
import org.springframework.context.ApplicationContext;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.yaml.snakeyaml.Yaml;

import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Flux;

/**
 * ConfigGateway
 */
@Component
@Slf4j
public class ConfigGateway implements RouteDefinitionLocator {

  List<MediaType> mediaTypes = new ArrayList<>();

  @Override
  public Flux<RouteDefinition> getRouteDefinitions() {

    Object inputStream = null;
    String[] gatewayConfigPathList = { "gateway.yml", "gateway.yaml", "Gateway.yml", "Gateway.yaml" };
    for (String gatewayConfigPath : gatewayConfigPathList) {
      inputStream = this.getClass().getClassLoader().getResourceAsStream(gatewayConfigPath);
      if (inputStream != null) {
        break;
      }
    }

    if (inputStream != null) {
      Map<String, Object> configYaml = new Yaml().load((InputStream) inputStream);
      if (configYaml != null) {

        // streaming-media-types 这个先不管
        // NettyWriteResponseFilter nwrf =
        // applicationContext.getBean(NettyWriteResponseFilter.class);
        // try {
        // Field field = nwrf.getClass().getDeclaredField("streamingMediaTypes");
        // field.setAccessible(true);
        // List<MediaType> mts = new ArrayList<MediaType>();
        // mts.add(MediaType.TEXT_EVENT_STREAM);
        // mts.add(MediaType.APPLICATION_STREAM_JSON);
        // field.set(nwrf, mts);
        // } catch (Exception e) {
        // log.error(e.toString());
        // }

        Map<String, Object> configDefault = (Map<String, Object>) configYaml.get("default");
        Map<String, Object> configDubbo = (Map<String, Object>) configYaml.get("dubbo");

        // ConfigProperties.createHttp2Dubbo(configDubbo);
        return Flux.fromIterable(ConfigProperties.getConfigDefault(configDefault));
      }
    }
    return null;
    // return Flux.fromIterable(it);
  }
}